#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Import/Export
- Name     : Shapes
- ID       : io_shapes

Description
----------
Tools for the import and export of vector data.
'''

from PySAGA.helper import Tool_Wrapper

def Export_GStat_Shapes(
    SHAPES=None,
    FILENAME=None, Verbose=2):
    '''
    Export GStat Shapes
    ----------
    [io_shapes.0]\n
    GStat shapes format export.\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - FILENAME [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '0', 'Export GStat Shapes')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('FILENAME', FILENAME)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_0(
    SHAPES=None,
    FILENAME=None, Verbose=2):
    '''
    Export GStat Shapes
    ----------
    [io_shapes.0]\n
    GStat shapes format export.\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - FILENAME [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '0', 'Export GStat Shapes')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('FILENAME', FILENAME)
        return Tool.Execute(Verbose)
    return False

def Import_GStat_Shapes(
    SHAPES=None,
    FILENAME=None, Verbose=2):
    '''
    Import GStat Shapes
    ----------
    [io_shapes.1]\n
    GStat shapes format import.\n
    Arguments
    ----------
    - SHAPES [`output shapes`] : Shapes
    - FILENAME [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '1', 'Import GStat Shapes')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('FILENAME', FILENAME)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_1(
    SHAPES=None,
    FILENAME=None, Verbose=2):
    '''
    Import GStat Shapes
    ----------
    [io_shapes.1]\n
    GStat shapes format import.\n
    Arguments
    ----------
    - SHAPES [`output shapes`] : Shapes
    - FILENAME [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '1', 'Import GStat Shapes')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('FILENAME', FILENAME)
        return Tool.Execute(Verbose)
    return False

def Export_Shapes_to_XYZ(
    POINTS=None,
    FIELD=None,
    HEADER=None,
    SEPARATE=None,
    FILENAME=None, Verbose=2):
    '''
    Export Shapes to XYZ
    ----------
    [io_shapes.2]\n
    XYZ export filter for shapes.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Shapes
    - FIELD [`table field`] : Attribute
    - HEADER [`boolean`] : Save Table Header. Default: 1
    - SEPARATE [`choice`] : Separate Line/Polygon Points. Available Choices: [0] none [1] * [2] number of points Default: 0
    - FILENAME [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '2', 'Export Shapes to XYZ')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('HEADER', HEADER)
        Tool.Set_Option('SEPARATE', SEPARATE)
        Tool.Set_Option('FILENAME', FILENAME)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_2(
    POINTS=None,
    FIELD=None,
    HEADER=None,
    SEPARATE=None,
    FILENAME=None, Verbose=2):
    '''
    Export Shapes to XYZ
    ----------
    [io_shapes.2]\n
    XYZ export filter for shapes.\n
    Arguments
    ----------
    - POINTS [`input shapes`] : Shapes
    - FIELD [`table field`] : Attribute
    - HEADER [`boolean`] : Save Table Header. Default: 1
    - SEPARATE [`choice`] : Separate Line/Polygon Points. Available Choices: [0] none [1] * [2] number of points Default: 0
    - FILENAME [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '2', 'Export Shapes to XYZ')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('HEADER', HEADER)
        Tool.Set_Option('SEPARATE', SEPARATE)
        Tool.Set_Option('FILENAME', FILENAME)
        return Tool.Execute(Verbose)
    return False

def Import_Shapes_from_XYZ(
    POINTS=None,
    HEADLINE=None,
    FILENAME=None,
    CRS_STRING=None, Verbose=2):
    '''
    Import Shapes from XYZ
    ----------
    [io_shapes.3]\n
    Imports points from a table with only list of x, y, z coordinates provided as simple text. If your table has a more complex structure, you should import it as table and then use the 'points from table' conversion tool.\n
    Arguments
    ----------
    - POINTS [`output shapes`] : Points
    - HEADLINE [`boolean`] : File contains headline. Default: 1
    - FILENAME [`file path`] : File
    - CRS_STRING [`text`] : Coordinate System Definition. Supported formats comprise PROJ and WKT strings, object codes (e.g. "EPSG:4326").

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '3', 'Import Shapes from XYZ')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('HEADLINE', HEADLINE)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('CRS_STRING', CRS_STRING)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_3(
    POINTS=None,
    HEADLINE=None,
    FILENAME=None,
    CRS_STRING=None, Verbose=2):
    '''
    Import Shapes from XYZ
    ----------
    [io_shapes.3]\n
    Imports points from a table with only list of x, y, z coordinates provided as simple text. If your table has a more complex structure, you should import it as table and then use the 'points from table' conversion tool.\n
    Arguments
    ----------
    - POINTS [`output shapes`] : Points
    - HEADLINE [`boolean`] : File contains headline. Default: 1
    - FILENAME [`file path`] : File
    - CRS_STRING [`text`] : Coordinate System Definition. Supported formats comprise PROJ and WKT strings, object codes (e.g. "EPSG:4326").

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '3', 'Import Shapes from XYZ')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('HEADLINE', HEADLINE)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('CRS_STRING', CRS_STRING)
        return Tool.Execute(Verbose)
    return False

def Export_Shapes_to_Generate(
    SHAPES=None,
    FIELD=None,
    FILE=None, Verbose=2):
    '''
    Export Shapes to Generate
    ----------
    [io_shapes.4]\n
    Export generate shapes format.\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - FIELD [`table field`] : Attribute
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '4', 'Export Shapes to Generate')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_4(
    SHAPES=None,
    FIELD=None,
    FILE=None, Verbose=2):
    '''
    Export Shapes to Generate
    ----------
    [io_shapes.4]\n
    Export generate shapes format.\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - FIELD [`table field`] : Attribute
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '4', 'Export Shapes to Generate')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def Export_Surfer_Blanking_File(
    SHAPES=None,
    NAME=None,
    DESC=None,
    ZVAL=None,
    FILE=None, Verbose=2):
    '''
    Export Surfer Blanking File
    ----------
    [io_shapes.5]\n
    Export shapes to Golden Software's Surfer Blanking File format.\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - NAME [`table field`] : Name
    - DESC [`table field`] : Description
    - ZVAL [`table field`] : z values
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '5', 'Export Surfer Blanking File')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('DESC', DESC)
        Tool.Set_Option('ZVAL', ZVAL)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_5(
    SHAPES=None,
    NAME=None,
    DESC=None,
    ZVAL=None,
    FILE=None, Verbose=2):
    '''
    Export Surfer Blanking File
    ----------
    [io_shapes.5]\n
    Export shapes to Golden Software's Surfer Blanking File format.\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - NAME [`table field`] : Name
    - DESC [`table field`] : Description
    - ZVAL [`table field`] : z values
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '5', 'Export Surfer Blanking File')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('DESC', DESC)
        Tool.Set_Option('ZVAL', ZVAL)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def Import_Surfer_Blanking_Files(
    SHAPES=None,
    TABLE=None,
    FILE=None,
    TYPE=None, Verbose=2):
    '''
    Import Surfer Blanking Files
    ----------
    [io_shapes.6]\n
    Import polygons/polylines from Golden Software's Surfer Blanking File format.\n
    Arguments
    ----------
    - SHAPES [`output shapes`] : Shapes
    - TABLE [`output table`] : Look up table (Points)
    - FILE [`file path`] : File
    - TYPE [`choice`] : Shape Type. Available Choices: [0] points [1] lines [2] polygons Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '6', 'Import Surfer Blanking Files')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('TYPE', TYPE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_6(
    SHAPES=None,
    TABLE=None,
    FILE=None,
    TYPE=None, Verbose=2):
    '''
    Import Surfer Blanking Files
    ----------
    [io_shapes.6]\n
    Import polygons/polylines from Golden Software's Surfer Blanking File format.\n
    Arguments
    ----------
    - SHAPES [`output shapes`] : Shapes
    - TABLE [`output table`] : Look up table (Points)
    - FILE [`file path`] : File
    - TYPE [`choice`] : Shape Type. Available Choices: [0] points [1] lines [2] polygons Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '6', 'Import Surfer Blanking Files')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('TYPE', TYPE)
        return Tool.Execute(Verbose)
    return False

def Export_Atlas_Boundary_File(
    SHAPES=None,
    PNAME=None,
    SNAME=None,
    FILE=None, Verbose=2):
    '''
    Export Atlas Boundary File
    ----------
    [io_shapes.7]\n
    Export Atlas Boundary File\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - PNAME [`table field`] : Primary Name
    - SNAME [`table field`] : Secondary Name
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '7', 'Export Atlas Boundary File')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('PNAME', PNAME)
        Tool.Set_Option('SNAME', SNAME)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_7(
    SHAPES=None,
    PNAME=None,
    SNAME=None,
    FILE=None, Verbose=2):
    '''
    Export Atlas Boundary File
    ----------
    [io_shapes.7]\n
    Export Atlas Boundary File\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - PNAME [`table field`] : Primary Name
    - SNAME [`table field`] : Secondary Name
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '7', 'Export Atlas Boundary File')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('PNAME', PNAME)
        Tool.Set_Option('SNAME', SNAME)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def Import_Atlas_Boundary_File(
    FILE=None, Verbose=2):
    '''
    Import Atlas Boundary File
    ----------
    [io_shapes.8]\n
    Import Atlas Boundary File\n
    Arguments
    ----------
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '8', 'Import Atlas Boundary File')
    if Tool.is_Okay():
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_8(
    FILE=None, Verbose=2):
    '''
    Import Atlas Boundary File
    ----------
    [io_shapes.8]\n
    Import Atlas Boundary File\n
    Arguments
    ----------
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '8', 'Import Atlas Boundary File')
    if Tool.is_Okay():
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def Export_WASP_terrain_map_file(
    SHAPES=None,
    ELEVATION=None,
    FILE=None, Verbose=2):
    '''
    Export WASP terrain map file
    ----------
    [io_shapes.9]\n
    Export WAsP (Wind Atlas Analysis and Application Program) terrain map file\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Contour Lines
    - ELEVATION [`table field`] : Map File
    - FILE [`file path`] : File Name

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '9', 'Export WASP terrain map file')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('ELEVATION', ELEVATION)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_9(
    SHAPES=None,
    ELEVATION=None,
    FILE=None, Verbose=2):
    '''
    Export WASP terrain map file
    ----------
    [io_shapes.9]\n
    Export WAsP (Wind Atlas Analysis and Application Program) terrain map file\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Contour Lines
    - ELEVATION [`table field`] : Map File
    - FILE [`file path`] : File Name

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '9', 'Export WASP terrain map file')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('ELEVATION', ELEVATION)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def Import_WASP_terrain_map_file(
    SHAPES=None,
    FILE=None,
    METHOD=None, Verbose=2):
    '''
    Import WASP terrain map file
    ----------
    [io_shapes.10]\n
    Import WAsP (Wind Atlas Analysis and Application Program) terrain map file\n
    Arguments
    ----------
    - SHAPES [`output shapes`] : Contour Lines
    - FILE [`file path`] : File Name
    - METHOD [`choice`] : Input Specification. Available Choices: [0] elevation [1] roughness [2] elevation and roughness Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '10', 'Import WASP terrain map file')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('METHOD', METHOD)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_10(
    SHAPES=None,
    FILE=None,
    METHOD=None, Verbose=2):
    '''
    Import WASP terrain map file
    ----------
    [io_shapes.10]\n
    Import WAsP (Wind Atlas Analysis and Application Program) terrain map file\n
    Arguments
    ----------
    - SHAPES [`output shapes`] : Contour Lines
    - FILE [`file path`] : File Name
    - METHOD [`choice`] : Input Specification. Available Choices: [0] elevation [1] roughness [2] elevation and roughness Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '10', 'Import WASP terrain map file')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('METHOD', METHOD)
        return Tool.Execute(Verbose)
    return False

def Import_Stereo_Lithography_File_STL(
    POINTS=None,
    POLYGONS=None,
    GRID=None,
    TIN=None,
    FILE=None,
    METHOD=None,
    CENTROIDS=None,
    DUPLICATES=None,
    GRID_DIM=None,
    GRID_WIDTH=None,
    GRID_SIZE=None,
    ROTATE=None,
    ROT_X=None,
    ROT_Y=None,
    ROT_Z=None, Verbose=2):
    '''
    Import Stereo Lithography File (STL)
    ----------
    [io_shapes.11]\n
    An StL ("StereoLithography") file is a triangular representation of a 3-dimensional surface geometry. The surface is tessellated or broken down logically into a series of small triangles (facets). Each facet is described by a perpendicular direction and three points representing the vertices (corners) of the triangle (Ennex Research Corporation). The StL file format is commonly used for 3D printing.\n
    Arguments
    ----------
    - POINTS [`output point cloud`] : Points
    - POLYGONS [`output shapes`] : Polygons
    - GRID [`output data object`] : Grid
    - TIN [`output TIN`] : TIN
    - FILE [`file path`] : File
    - METHOD [`choice`] : Target. Available Choices: [0] Points [1] Polygons [2] TIN [3] Grid Default: 2
    - CENTROIDS [`boolean`] : Centroids. Default: 0
    - DUPLICATES [`boolean`] : Remove Duplicates. Default: 1
    - GRID_DIM [`choice`] : Grid Dimension. Available Choices: [0] Width [1] Cellsize Default: 0
    - GRID_WIDTH [`integer number`] : Width. Minimum: 10 Default: 2000 Number of cells.
    - GRID_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - ROTATE [`boolean`] : Rotation. Default: 0
    - ROT_X [`floating point number`] : X Axis. Default: 0.000000
    - ROT_Y [`floating point number`] : Y Axis. Default: 0.000000
    - ROT_Z [`floating point number`] : Z Axis. Default: 0.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '11', 'Import Stereo Lithography File (STL)')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Output('POLYGONS', POLYGONS)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Output('TIN', TIN)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('CENTROIDS', CENTROIDS)
        Tool.Set_Option('DUPLICATES', DUPLICATES)
        Tool.Set_Option('GRID_DIM', GRID_DIM)
        Tool.Set_Option('GRID_WIDTH', GRID_WIDTH)
        Tool.Set_Option('GRID_SIZE', GRID_SIZE)
        Tool.Set_Option('ROTATE', ROTATE)
        Tool.Set_Option('ROT_X', ROT_X)
        Tool.Set_Option('ROT_Y', ROT_Y)
        Tool.Set_Option('ROT_Z', ROT_Z)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_11(
    POINTS=None,
    POLYGONS=None,
    GRID=None,
    TIN=None,
    FILE=None,
    METHOD=None,
    CENTROIDS=None,
    DUPLICATES=None,
    GRID_DIM=None,
    GRID_WIDTH=None,
    GRID_SIZE=None,
    ROTATE=None,
    ROT_X=None,
    ROT_Y=None,
    ROT_Z=None, Verbose=2):
    '''
    Import Stereo Lithography File (STL)
    ----------
    [io_shapes.11]\n
    An StL ("StereoLithography") file is a triangular representation of a 3-dimensional surface geometry. The surface is tessellated or broken down logically into a series of small triangles (facets). Each facet is described by a perpendicular direction and three points representing the vertices (corners) of the triangle (Ennex Research Corporation). The StL file format is commonly used for 3D printing.\n
    Arguments
    ----------
    - POINTS [`output point cloud`] : Points
    - POLYGONS [`output shapes`] : Polygons
    - GRID [`output data object`] : Grid
    - TIN [`output TIN`] : TIN
    - FILE [`file path`] : File
    - METHOD [`choice`] : Target. Available Choices: [0] Points [1] Polygons [2] TIN [3] Grid Default: 2
    - CENTROIDS [`boolean`] : Centroids. Default: 0
    - DUPLICATES [`boolean`] : Remove Duplicates. Default: 1
    - GRID_DIM [`choice`] : Grid Dimension. Available Choices: [0] Width [1] Cellsize Default: 0
    - GRID_WIDTH [`integer number`] : Width. Minimum: 10 Default: 2000 Number of cells.
    - GRID_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - ROTATE [`boolean`] : Rotation. Default: 0
    - ROT_X [`floating point number`] : X Axis. Default: 0.000000
    - ROT_Y [`floating point number`] : Y Axis. Default: 0.000000
    - ROT_Z [`floating point number`] : Z Axis. Default: 0.000000

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '11', 'Import Stereo Lithography File (STL)')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Output('POLYGONS', POLYGONS)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Output('TIN', TIN)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('METHOD', METHOD)
        Tool.Set_Option('CENTROIDS', CENTROIDS)
        Tool.Set_Option('DUPLICATES', DUPLICATES)
        Tool.Set_Option('GRID_DIM', GRID_DIM)
        Tool.Set_Option('GRID_WIDTH', GRID_WIDTH)
        Tool.Set_Option('GRID_SIZE', GRID_SIZE)
        Tool.Set_Option('ROTATE', ROTATE)
        Tool.Set_Option('ROT_X', ROT_X)
        Tool.Set_Option('ROT_Y', ROT_Y)
        Tool.Set_Option('ROT_Z', ROT_Z)
        return Tool.Execute(Verbose)
    return False

def Export_TIN_to_Stereo_Lithography_File_STL(
    TIN=None,
    ZFIELD=None,
    FILE=None,
    BINARY=None, Verbose=2):
    '''
    Export TIN to Stereo Lithography File (STL)
    ----------
    [io_shapes.12]\n
    An StL ("StereoLithography") file is a triangular representation of a 3-dimensional surface geometry. The surface is tessellated or broken down logically into a series of small triangles (facets). Each facet is described by a perpendicular direction and three points representing the vertices (corners) of the triangle (Ennex Research Corporation). The StL file format is commonly used for 3D printing.\n
    Arguments
    ----------
    - TIN [`input TIN`] : TIN
    - ZFIELD [`table field`] : Attribute
    - FILE [`file path`] : File
    - BINARY [`choice`] : Output Type. Available Choices: [0] ASCII [1] binary Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '12', 'Export TIN to Stereo Lithography File (STL)')
    if Tool.is_Okay():
        Tool.Set_Input ('TIN', TIN)
        Tool.Set_Option('ZFIELD', ZFIELD)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('BINARY', BINARY)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_12(
    TIN=None,
    ZFIELD=None,
    FILE=None,
    BINARY=None, Verbose=2):
    '''
    Export TIN to Stereo Lithography File (STL)
    ----------
    [io_shapes.12]\n
    An StL ("StereoLithography") file is a triangular representation of a 3-dimensional surface geometry. The surface is tessellated or broken down logically into a series of small triangles (facets). Each facet is described by a perpendicular direction and three points representing the vertices (corners) of the triangle (Ennex Research Corporation). The StL file format is commonly used for 3D printing.\n
    Arguments
    ----------
    - TIN [`input TIN`] : TIN
    - ZFIELD [`table field`] : Attribute
    - FILE [`file path`] : File
    - BINARY [`choice`] : Output Type. Available Choices: [0] ASCII [1] binary Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '12', 'Export TIN to Stereo Lithography File (STL)')
    if Tool.is_Okay():
        Tool.Set_Input ('TIN', TIN)
        Tool.Set_Option('ZFIELD', ZFIELD)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('BINARY', BINARY)
        return Tool.Execute(Verbose)
    return False

def Import_GPX(
    SHAPES=None,
    FILE=None,
    TIME=None, Verbose=2):
    '''
    Import GPX
    ----------
    [io_shapes.13]\n
    Imports GPS data from GPS eXchange format GPX.\n
    References:\n
    [The GPS Exchange Format](http://www.topografix.com/)\n
    Arguments
    ----------
    - SHAPES [`output shapes list`] : GPX Import
    - FILE [`file path`] : File
    - TIME [`boolean`] : Time Stamp without date. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '13', 'Import GPX')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('TIME', TIME)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_13(
    SHAPES=None,
    FILE=None,
    TIME=None, Verbose=2):
    '''
    Import GPX
    ----------
    [io_shapes.13]\n
    Imports GPS data from GPS eXchange format GPX.\n
    References:\n
    [The GPS Exchange Format](http://www.topografix.com/)\n
    Arguments
    ----------
    - SHAPES [`output shapes list`] : GPX Import
    - FILE [`file path`] : File
    - TIME [`boolean`] : Time Stamp without date. Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '13', 'Import GPX')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('TIME', TIME)
        return Tool.Execute(Verbose)
    return False

def Export_GPX(
    SHAPES=None,
    FILE=None,
    ELE=None,
    NAME=None,
    CMT=None,
    DESC=None, Verbose=2):
    '''
    Export GPX
    ----------
    [io_shapes.14]\n
    Exports vector data points to GPS eXchange format GPX.\n
    References:\n
    [The GPS Exchange Format](http://www.topografix.com/)\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - FILE [`file path`] : File
    - ELE [`table field`] : Elevation
    - NAME [`table field`] : Name
    - CMT [`table field`] : Comment
    - DESC [`table field`] : Description

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '14', 'Export GPX')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('ELE', ELE)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('CMT', CMT)
        Tool.Set_Option('DESC', DESC)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_14(
    SHAPES=None,
    FILE=None,
    ELE=None,
    NAME=None,
    CMT=None,
    DESC=None, Verbose=2):
    '''
    Export GPX
    ----------
    [io_shapes.14]\n
    Exports vector data points to GPS eXchange format GPX.\n
    References:\n
    [The GPS Exchange Format](http://www.topografix.com/)\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - FILE [`file path`] : File
    - ELE [`table field`] : Elevation
    - NAME [`table field`] : Name
    - CMT [`table field`] : Comment
    - DESC [`table field`] : Description

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '14', 'Export GPX')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('ELE', ELE)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('CMT', CMT)
        Tool.Set_Option('DESC', DESC)
        return Tool.Execute(Verbose)
    return False

def Import_Point_Cloud_from_Shape_File(
    POINTS=None,
    FILE=None, Verbose=2):
    '''
    Import Point Cloud from Shape File
    ----------
    [io_shapes.15]\n
    Imports a point cloud from a point shapefile.\n
    Arguments
    ----------
    - POINTS [`output point cloud`] : Point Cloud
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '15', 'Import Point Cloud from Shape File')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_15(
    POINTS=None,
    FILE=None, Verbose=2):
    '''
    Import Point Cloud from Shape File
    ----------
    [io_shapes.15]\n
    Imports a point cloud from a point shapefile.\n
    Arguments
    ----------
    - POINTS [`output point cloud`] : Point Cloud
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '15', 'Import Point Cloud from Shape File')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def Import_Point_Cloud_from_Text_File(
    POINTS=None,
    FILE=None,
    SEPARATOR=None,
    SKIP_HEADER=None,
    XFIELD=None,
    YFIELD=None,
    ZFIELD=None,
    FIELDS=None,
    FIELDNAMES=None,
    FIELDTYPES=None,
    CRS_STRING=None, Verbose=2):
    '''
    Import Point Cloud from Text File
    ----------
    [io_shapes.16]\n
    Creates a point cloud from a text file.\n
    The input file must have at least three columns holding the x, y, z coordinates of each point. You must specify the field index (i.e. the column) of these. Field index starts to count with 1. In case you like to import additional attributes, you have to provide the field indexes for those attributes with the -FIELDS option as integer numbers separated by semicolon (e.g. "-FIELDS=4;5;8").\n
    You have also to select the field separator that is used by the file and if the first line of the file should be skipped (in case it contains column headings).\n
    The columns in the input file can be in any order, and you can omit columns, but you have to provide the correct field index for those fields that you like to import.\n
    The tool usage differs slightly between SAGA GUI and SAGA CMD. With SAGA GUI you can specify names and types for additional fields in the 'Specifications' sub dialog. To do this using SAGA CMD you have to use the -FIELDNAMES and -FIELDTYPES options. The first one is for the field names, the second for the data type specification (see the GUI which number equals which data type). Again entries have to be separated by semicolons, e.g. "-FIELDNAMES=intensity;class;range -FIELDTYPES=2;2;3".\n
    Arguments
    ----------
    - POINTS [`output point cloud`] : Point Cloud
    - FILE [`file path`] : Text File
    - SEPARATOR [`choice`] : Field Separator. Available Choices: [0] tabulator [1] space [2] comma [3] semicolon Default: 0 Field Separator
    - SKIP_HEADER [`boolean`] : Skip first line. Default: 0 Skip first line as it contains column names.
    - XFIELD [`integer number`] : X is Column .... Minimum: 1 Default: 1 The column holding the X-coordinate.
    - YFIELD [`integer number`] : Y is Column .... Minimum: 1 Default: 2 The column holding the Y-coordinate.
    - ZFIELD [`integer number`] : Z is Column .... Minimum: 1 Default: 3 The column holding the Z-coordinate.
    - FIELDS [`text`] : Fields. The index (starting with 1) of the fields to import, separated by semicolon, e.g. "5;6;8"
    - FIELDNAMES [`text`] : Field Names. The name to use for each field, separated by semicolon, e.g. "intensity;class;range"
    - FIELDTYPES [`text`] : Field Types. The datatype to use for each field, separated by semicolon, e.g. "2;2;3;". The number equals the choice selection, see GUI version.
    - CRS_STRING [`text`] : Coordinate System Definition. Supported formats comprise PROJ and WKT strings, object codes (e.g. "EPSG:4326").

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '16', 'Import Point Cloud from Text File')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('SEPARATOR', SEPARATOR)
        Tool.Set_Option('SKIP_HEADER', SKIP_HEADER)
        Tool.Set_Option('XFIELD', XFIELD)
        Tool.Set_Option('YFIELD', YFIELD)
        Tool.Set_Option('ZFIELD', ZFIELD)
        Tool.Set_Option('FIELDS', FIELDS)
        Tool.Set_Option('FIELDNAMES', FIELDNAMES)
        Tool.Set_Option('FIELDTYPES', FIELDTYPES)
        Tool.Set_Option('CRS_STRING', CRS_STRING)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_16(
    POINTS=None,
    FILE=None,
    SEPARATOR=None,
    SKIP_HEADER=None,
    XFIELD=None,
    YFIELD=None,
    ZFIELD=None,
    FIELDS=None,
    FIELDNAMES=None,
    FIELDTYPES=None,
    CRS_STRING=None, Verbose=2):
    '''
    Import Point Cloud from Text File
    ----------
    [io_shapes.16]\n
    Creates a point cloud from a text file.\n
    The input file must have at least three columns holding the x, y, z coordinates of each point. You must specify the field index (i.e. the column) of these. Field index starts to count with 1. In case you like to import additional attributes, you have to provide the field indexes for those attributes with the -FIELDS option as integer numbers separated by semicolon (e.g. "-FIELDS=4;5;8").\n
    You have also to select the field separator that is used by the file and if the first line of the file should be skipped (in case it contains column headings).\n
    The columns in the input file can be in any order, and you can omit columns, but you have to provide the correct field index for those fields that you like to import.\n
    The tool usage differs slightly between SAGA GUI and SAGA CMD. With SAGA GUI you can specify names and types for additional fields in the 'Specifications' sub dialog. To do this using SAGA CMD you have to use the -FIELDNAMES and -FIELDTYPES options. The first one is for the field names, the second for the data type specification (see the GUI which number equals which data type). Again entries have to be separated by semicolons, e.g. "-FIELDNAMES=intensity;class;range -FIELDTYPES=2;2;3".\n
    Arguments
    ----------
    - POINTS [`output point cloud`] : Point Cloud
    - FILE [`file path`] : Text File
    - SEPARATOR [`choice`] : Field Separator. Available Choices: [0] tabulator [1] space [2] comma [3] semicolon Default: 0 Field Separator
    - SKIP_HEADER [`boolean`] : Skip first line. Default: 0 Skip first line as it contains column names.
    - XFIELD [`integer number`] : X is Column .... Minimum: 1 Default: 1 The column holding the X-coordinate.
    - YFIELD [`integer number`] : Y is Column .... Minimum: 1 Default: 2 The column holding the Y-coordinate.
    - ZFIELD [`integer number`] : Z is Column .... Minimum: 1 Default: 3 The column holding the Z-coordinate.
    - FIELDS [`text`] : Fields. The index (starting with 1) of the fields to import, separated by semicolon, e.g. "5;6;8"
    - FIELDNAMES [`text`] : Field Names. The name to use for each field, separated by semicolon, e.g. "intensity;class;range"
    - FIELDTYPES [`text`] : Field Types. The datatype to use for each field, separated by semicolon, e.g. "2;2;3;". The number equals the choice selection, see GUI version.
    - CRS_STRING [`text`] : Coordinate System Definition. Supported formats comprise PROJ and WKT strings, object codes (e.g. "EPSG:4326").

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '16', 'Import Point Cloud from Text File')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('SEPARATOR', SEPARATOR)
        Tool.Set_Option('SKIP_HEADER', SKIP_HEADER)
        Tool.Set_Option('XFIELD', XFIELD)
        Tool.Set_Option('YFIELD', YFIELD)
        Tool.Set_Option('ZFIELD', ZFIELD)
        Tool.Set_Option('FIELDS', FIELDS)
        Tool.Set_Option('FIELDNAMES', FIELDNAMES)
        Tool.Set_Option('FIELDTYPES', FIELDTYPES)
        Tool.Set_Option('CRS_STRING', CRS_STRING)
        return Tool.Execute(Verbose)
    return False

def Export_Scalable_Vector_Graphics_SVG_File(
    LAYERS=None,
    LAYER=None,
    OUTPUT=None,
    FIELD=None,
    FILE=None, Verbose=2):
    '''
    Export Scalable Vector Graphics (SVG) File
    ----------
    [io_shapes.17]\n
    Export shapes to Scalable Vector Graphics (SVG) File.\n
    Arguments
    ----------
    - LAYERS [`input shapes list`] : Layers
    - LAYER [`input shapes`] : Layer
    - OUTPUT [`choice`] : Output. Available Choices: [0] single layer [1] multiple layers Default: 1
    - FIELD [`table field`] : Attribute
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '17', 'Export Scalable Vector Graphics (SVG) File')
    if Tool.is_Okay():
        Tool.Set_Input ('LAYERS', LAYERS)
        Tool.Set_Input ('LAYER', LAYER)
        Tool.Set_Option('OUTPUT', OUTPUT)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_17(
    LAYERS=None,
    LAYER=None,
    OUTPUT=None,
    FIELD=None,
    FILE=None, Verbose=2):
    '''
    Export Scalable Vector Graphics (SVG) File
    ----------
    [io_shapes.17]\n
    Export shapes to Scalable Vector Graphics (SVG) File.\n
    Arguments
    ----------
    - LAYERS [`input shapes list`] : Layers
    - LAYER [`input shapes`] : Layer
    - OUTPUT [`choice`] : Output. Available Choices: [0] single layer [1] multiple layers Default: 1
    - FIELD [`table field`] : Attribute
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '17', 'Export Scalable Vector Graphics (SVG) File')
    if Tool.is_Okay():
        Tool.Set_Input ('LAYERS', LAYERS)
        Tool.Set_Input ('LAYER', LAYER)
        Tool.Set_Option('OUTPUT', OUTPUT)
        Tool.Set_Option('FIELD', FIELD)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def Export_Point_Cloud_to_Text_File(
    POINTS=None,
    FILE=None,
    WRITE_HEADER=None,
    FIELDSEP=None,
    FIELDS=None,
    PRECISIONS=None, Verbose=2):
    '''
    Export Point Cloud to Text File
    ----------
    [io_shapes.18]\n
    Exports a point cloud to a text file. Once the tool is executed, a pop-up dialog allows one to specify the fields to be exported and their decimal precision.\n
    Tool usage is different between SAGA GUI and SAGA CMD: With SAGA GUI you will get prompted to choose the fields to export and the decimal precisions to use once you execute the tool. With SAGA CMD you have to provide two strings with the -FIELDS and -PRECISIONS parameters. The first one must contain the field numbers, the latter the precisions (separated by semicolon). Field numbers start with 1, e.g. -FIELDS="1;2;3;5" -PRECISIONS="2;2;2;0".\n
    Arguments
    ----------
    - POINTS [`input point cloud`] : Point Cloud. The point cloud to export.
    - FILE [`file path`] : Text File. The file to write the point cloud to.
    - WRITE_HEADER [`boolean`] : Write Header. Default: 0 Write column names.
    - FIELDSEP [`choice`] : Field Separator. Available Choices: [0] tabulator [1] space [2] comma Default: 0 Field Separator
    - FIELDS [`text`] : Fields. The numbers (starting from 1) of the fields to export, separated by semicolon, e.g. "1;2;3;5"
    - PRECISIONS [`text`] : Precisions. The decimal precision to use for each field, separated by semicolon, e.g. "2;2;2;0"

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '18', 'Export Point Cloud to Text File')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('WRITE_HEADER', WRITE_HEADER)
        Tool.Set_Option('FIELDSEP', FIELDSEP)
        Tool.Set_Option('FIELDS', FIELDS)
        Tool.Set_Option('PRECISIONS', PRECISIONS)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_18(
    POINTS=None,
    FILE=None,
    WRITE_HEADER=None,
    FIELDSEP=None,
    FIELDS=None,
    PRECISIONS=None, Verbose=2):
    '''
    Export Point Cloud to Text File
    ----------
    [io_shapes.18]\n
    Exports a point cloud to a text file. Once the tool is executed, a pop-up dialog allows one to specify the fields to be exported and their decimal precision.\n
    Tool usage is different between SAGA GUI and SAGA CMD: With SAGA GUI you will get prompted to choose the fields to export and the decimal precisions to use once you execute the tool. With SAGA CMD you have to provide two strings with the -FIELDS and -PRECISIONS parameters. The first one must contain the field numbers, the latter the precisions (separated by semicolon). Field numbers start with 1, e.g. -FIELDS="1;2;3;5" -PRECISIONS="2;2;2;0".\n
    Arguments
    ----------
    - POINTS [`input point cloud`] : Point Cloud. The point cloud to export.
    - FILE [`file path`] : Text File. The file to write the point cloud to.
    - WRITE_HEADER [`boolean`] : Write Header. Default: 0 Write column names.
    - FIELDSEP [`choice`] : Field Separator. Available Choices: [0] tabulator [1] space [2] comma Default: 0 Field Separator
    - FIELDS [`text`] : Fields. The numbers (starting from 1) of the fields to export, separated by semicolon, e.g. "1;2;3;5"
    - PRECISIONS [`text`] : Precisions. The decimal precision to use for each field, separated by semicolon, e.g. "2;2;2;0"

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '18', 'Export Point Cloud to Text File')
    if Tool.is_Okay():
        Tool.Set_Input ('POINTS', POINTS)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('WRITE_HEADER', WRITE_HEADER)
        Tool.Set_Option('FIELDSEP', FIELDSEP)
        Tool.Set_Option('FIELDS', FIELDS)
        Tool.Set_Option('PRECISIONS', PRECISIONS)
        return Tool.Execute(Verbose)
    return False

def Import_Simple_Features_from_Well_Known_Text(
    SHAPES=None,
    FILE=None,
    WKT=None, Verbose=2):
    '''
    Import Simple Features from Well Known Text
    ----------
    [io_shapes.19]\n
    Imports vector data from 'well known text' (WKT) simple features format.\n
    This import tool assumes that all features in a file are of the same type.\n
    Instead of importing from file(s), the tool also supports the conversion from a string provided with the 'WKT String' parameter.\n
    References:\n
    [Open Geospatial Consortium](http://www.opengeospatial.org/)\n
    Arguments
    ----------
    - SHAPES [`output shapes list`] : WKT Import
    - FILE [`file path`] : File
    - WKT [`long text`] : WKT String. Import WKT from string instead of file. Just paste the WKT.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '19', 'Import Simple Features from Well Known Text')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('WKT', WKT)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_19(
    SHAPES=None,
    FILE=None,
    WKT=None, Verbose=2):
    '''
    Import Simple Features from Well Known Text
    ----------
    [io_shapes.19]\n
    Imports vector data from 'well known text' (WKT) simple features format.\n
    This import tool assumes that all features in a file are of the same type.\n
    Instead of importing from file(s), the tool also supports the conversion from a string provided with the 'WKT String' parameter.\n
    References:\n
    [Open Geospatial Consortium](http://www.opengeospatial.org/)\n
    Arguments
    ----------
    - SHAPES [`output shapes list`] : WKT Import
    - FILE [`file path`] : File
    - WKT [`long text`] : WKT String. Import WKT from string instead of file. Just paste the WKT.

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '19', 'Import Simple Features from Well Known Text')
    if Tool.is_Okay():
        Tool.Set_Output('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('WKT', WKT)
        return Tool.Execute(Verbose)
    return False

def Export_Simple_Features_to_Well_Known_Text(
    SHAPES=None,
    FILE=None, Verbose=2):
    '''
    Export Simple Features to Well Known Text
    ----------
    [io_shapes.20]\n
    Exports vector data to 'well known text' (WKT) simple features format.\n
    References:\n
    [Open Geospatial Consortium](http://www.opengeospatial.org/)\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '20', 'Export Simple Features to Well Known Text')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_20(
    SHAPES=None,
    FILE=None, Verbose=2):
    '''
    Export Simple Features to Well Known Text
    ----------
    [io_shapes.20]\n
    Exports vector data to 'well known text' (WKT) simple features format.\n
    References:\n
    [Open Geospatial Consortium](http://www.opengeospatial.org/)\n
    Arguments
    ----------
    - SHAPES [`input shapes`] : Shapes
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '20', 'Export Simple Features to Well Known Text')
    if Tool.is_Okay():
        Tool.Set_Input ('SHAPES', SHAPES)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def Import_Building_Sketches_from_CityGML(
    BUILDINGS=None,
    FILES=None,
    PARTS=None, Verbose=2):
    '''
    Import Building Sketches from CityGML
    ----------
    [io_shapes.21]\n
    This tool facilitates the import of building sketches using a CityGML based file format, that is commonly used by German land surveying offices and geoinformation distributors.\n
    Arguments
    ----------
    - BUILDINGS [`output shapes`] : Buildings
    - FILES [`file path`] : Files
    - PARTS [`boolean`] : Check for Building Parts. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '21', 'Import Building Sketches from CityGML')
    if Tool.is_Okay():
        Tool.Set_Output('BUILDINGS', BUILDINGS)
        Tool.Set_Option('FILES', FILES)
        Tool.Set_Option('PARTS', PARTS)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_21(
    BUILDINGS=None,
    FILES=None,
    PARTS=None, Verbose=2):
    '''
    Import Building Sketches from CityGML
    ----------
    [io_shapes.21]\n
    This tool facilitates the import of building sketches using a CityGML based file format, that is commonly used by German land surveying offices and geoinformation distributors.\n
    Arguments
    ----------
    - BUILDINGS [`output shapes`] : Buildings
    - FILES [`file path`] : Files
    - PARTS [`boolean`] : Check for Building Parts. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '21', 'Import Building Sketches from CityGML')
    if Tool.is_Okay():
        Tool.Set_Output('BUILDINGS', BUILDINGS)
        Tool.Set_Option('FILES', FILES)
        Tool.Set_Option('PARTS', PARTS)
        return Tool.Execute(Verbose)
    return False

def Export_Polygons_to_HTML_Image_Map(
    POLYGONS=None,
    LINK=None,
    TITLE=None,
    IMAGE=None,
    FILE=None,
    LINK_PREFIX=None,
    LINK_SUFFIX=None, Verbose=2):
    '''
    Export Polygons to HTML Image Map
    ----------
    [io_shapes.22]\n
    Tool to create an HTML ImageMap with polygon dataset and image. The tool requires a polygon dataset and an georeferenced image file. It outputs an html file for possible further editing. The image name in the mapfile html is hard coded to be map.png.\n
    [How to build the link:]\n
    (-) [Link] Select link identifier from the attribute table of the polygon data layer.\n
    (-) [Prefix] Adds a prefix to the link identifier. If just for icon with alternative text and, add ''#'').\n
    (-) [Suffix] Adds a suffix of the link identifier. Likewise ''.txt, .html, .doc, .png''.\n
    The link will be built with the pattern ''Prefix + Identifier + Suffix''.\n
    [Take care!:]\n
    (-) Image and Polygon dataset have to share the same projection.(-) Special characters in link (like &ouml;, &szlig, oder &raquo;) are not automatically replaced to html entities. Replace them after creation in editor if needed.\n
    [Example for GUI usage:]\n
    (-) Load your polygon dataset and open it in a map.(-) Create an image of this map using the map's menu command ''Save Map to Workspace'' and save it to an image file using the file name ''map.png''.(-) Run this tool and save the output to the same directory as the image.(-) Open the html file in a browser for preview.(-) Further editing of the html file is suggested.\n
    Arguments
    ----------
    - POLYGONS [`input shapes`] : Polygons
    - LINK [`table field`] : Link
    - TITLE [`table field`] : Title
    - IMAGE [`grid system`] : Image. grid system of georeferenced image
    - FILE [`file path`] : File
    - LINK_PREFIX [`text`] : Link Prefix. Default: http://www.saga-gis.org/
    - LINK_SUFFIX [`text`] : Link Suffix. Default: index.html

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '22', 'Export Polygons to HTML Image Map')
    if Tool.is_Okay():
        Tool.Set_Input ('POLYGONS', POLYGONS)
        Tool.Set_Option('LINK', LINK)
        Tool.Set_Option('TITLE', TITLE)
        Tool.Set_Option('IMAGE', IMAGE)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('LINK_PREFIX', LINK_PREFIX)
        Tool.Set_Option('LINK_SUFFIX', LINK_SUFFIX)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_22(
    POLYGONS=None,
    LINK=None,
    TITLE=None,
    IMAGE=None,
    FILE=None,
    LINK_PREFIX=None,
    LINK_SUFFIX=None, Verbose=2):
    '''
    Export Polygons to HTML Image Map
    ----------
    [io_shapes.22]\n
    Tool to create an HTML ImageMap with polygon dataset and image. The tool requires a polygon dataset and an georeferenced image file. It outputs an html file for possible further editing. The image name in the mapfile html is hard coded to be map.png.\n
    [How to build the link:]\n
    (-) [Link] Select link identifier from the attribute table of the polygon data layer.\n
    (-) [Prefix] Adds a prefix to the link identifier. If just for icon with alternative text and, add ''#'').\n
    (-) [Suffix] Adds a suffix of the link identifier. Likewise ''.txt, .html, .doc, .png''.\n
    The link will be built with the pattern ''Prefix + Identifier + Suffix''.\n
    [Take care!:]\n
    (-) Image and Polygon dataset have to share the same projection.(-) Special characters in link (like &ouml;, &szlig, oder &raquo;) are not automatically replaced to html entities. Replace them after creation in editor if needed.\n
    [Example for GUI usage:]\n
    (-) Load your polygon dataset and open it in a map.(-) Create an image of this map using the map's menu command ''Save Map to Workspace'' and save it to an image file using the file name ''map.png''.(-) Run this tool and save the output to the same directory as the image.(-) Open the html file in a browser for preview.(-) Further editing of the html file is suggested.\n
    Arguments
    ----------
    - POLYGONS [`input shapes`] : Polygons
    - LINK [`table field`] : Link
    - TITLE [`table field`] : Title
    - IMAGE [`grid system`] : Image. grid system of georeferenced image
    - FILE [`file path`] : File
    - LINK_PREFIX [`text`] : Link Prefix. Default: http://www.saga-gis.org/
    - LINK_SUFFIX [`text`] : Link Suffix. Default: index.html

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '22', 'Export Polygons to HTML Image Map')
    if Tool.is_Okay():
        Tool.Set_Input ('POLYGONS', POLYGONS)
        Tool.Set_Option('LINK', LINK)
        Tool.Set_Option('TITLE', TITLE)
        Tool.Set_Option('IMAGE', IMAGE)
        Tool.Set_Option('FILE', FILE)
        Tool.Set_Option('LINK_PREFIX', LINK_PREFIX)
        Tool.Set_Option('LINK_SUFFIX', LINK_SUFFIX)
        return Tool.Execute(Verbose)
    return False

def Import_Point_Cloud_from_PTS_File(
    POINTS=None,
    FILENAME=None,
    RGB=None,
    CRS_STRING=None, Verbose=2):
    '''
    Import Point Cloud from PTS File
    ----------
    [io_shapes.23]\n
    Imports point cloud data from a PTS file.\n
    Arguments
    ----------
    - POINTS [`output point cloud`] : Points
    - FILENAME [`file path`] : File
    - RGB [`choice`] : Import RGB Values as.... Available Choices: [0] separate values [1] single rgb-coded integer value Default: 1
    - CRS_STRING [`text`] : Coordinate System Definition. Supported formats comprise PROJ and WKT strings, object codes (e.g. "EPSG:4326").

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '23', 'Import Point Cloud from PTS File')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('RGB', RGB)
        Tool.Set_Option('CRS_STRING', CRS_STRING)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_23(
    POINTS=None,
    FILENAME=None,
    RGB=None,
    CRS_STRING=None, Verbose=2):
    '''
    Import Point Cloud from PTS File
    ----------
    [io_shapes.23]\n
    Imports point cloud data from a PTS file.\n
    Arguments
    ----------
    - POINTS [`output point cloud`] : Points
    - FILENAME [`file path`] : File
    - RGB [`choice`] : Import RGB Values as.... Available Choices: [0] separate values [1] single rgb-coded integer value Default: 1
    - CRS_STRING [`text`] : Coordinate System Definition. Supported formats comprise PROJ and WKT strings, object codes (e.g. "EPSG:4326").

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '23', 'Import Point Cloud from PTS File')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILENAME', FILENAME)
        Tool.Set_Option('RGB', RGB)
        Tool.Set_Option('CRS_STRING', CRS_STRING)
        return Tool.Execute(Verbose)
    return False

def Import_ASEGGDF(
    POINTS=None,
    FILE=None, Verbose=2):
    '''
    Import ASEG-GDF
    ----------
    [io_shapes.24]\n
    Import a 'General Data Format Revision 2 (GDF2)' file as defined by the 'Australian Society of Exploration Geophysicists (ASEG)' as table.\n
    Arguments
    ----------
    - POINTS [`output shapes`] : Points
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '24', 'Import ASEG-GDF')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False

def run_tool_io_shapes_24(
    POINTS=None,
    FILE=None, Verbose=2):
    '''
    Import ASEG-GDF
    ----------
    [io_shapes.24]\n
    Import a 'General Data Format Revision 2 (GDF2)' file as defined by the 'Australian Society of Exploration Geophysicists (ASEG)' as table.\n
    Arguments
    ----------
    - POINTS [`output shapes`] : Points
    - FILE [`file path`] : File

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('io_shapes', '24', 'Import ASEG-GDF')
    if Tool.is_Okay():
        Tool.Set_Output('POINTS', POINTS)
        Tool.Set_Option('FILE', FILE)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Import/Export
- Name     : ODBC/OTL
- ID       : db_odbc

Description
----------
Database access via Open Data Base Connection (ODBC) interface. Based on the OTL (Oracle, Odbc and DB2-CLI Template Library), Version 4.0: [http://otl.sourceforge.net/](http://otl.sourceforge.net/)
'''

from PySAGA.helper import Tool_Wrapper

def Connect_to_ODBC_Source(
    DSN=None,
    USER=None,
    PASSWORD=None, Verbose=2):
    '''
    Connect to ODBC Source
    ----------
    [db_odbc.0]\n
    Connects to an ODBC source.\n
    Arguments
    ----------
    - DSN [`choice`] : Data Source. Available Choices: [0] The ODBC Data Source Name. Default: 0
    - USER [`text`] : User
    - PASSWORD [`text`] : Password

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '0', 'Connect to ODBC Source')
    if Tool.is_Okay():
        Tool.Set_Option('DSN', DSN)
        Tool.Set_Option('USER', USER)
        Tool.Set_Option('PASSWORD', PASSWORD)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_0(
    DSN=None,
    USER=None,
    PASSWORD=None, Verbose=2):
    '''
    Connect to ODBC Source
    ----------
    [db_odbc.0]\n
    Connects to an ODBC source.\n
    Arguments
    ----------
    - DSN [`choice`] : Data Source. Available Choices: [0] The ODBC Data Source Name. Default: 0
    - USER [`text`] : User
    - PASSWORD [`text`] : Password

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '0', 'Connect to ODBC Source')
    if Tool.is_Okay():
        Tool.Set_Option('DSN', DSN)
        Tool.Set_Option('USER', USER)
        Tool.Set_Option('PASSWORD', PASSWORD)
        return Tool.Execute(Verbose)
    return False

def Disconnect_from_ODBC_Source(
    CONNECTION=None,
    TRANSACT=None, Verbose=2):
    '''
    Disconnect from ODBC Source
    ----------
    [db_odbc.1]\n
    Disconnects an ODBC source connection.\n
    Arguments
    ----------
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TRANSACT [`choice`] : Transactions. Available Choices: [0] rollback [1] commit Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '1', 'Disconnect from ODBC Source')
    if Tool.is_Okay():
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TRANSACT', TRANSACT)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_1(
    CONNECTION=None,
    TRANSACT=None, Verbose=2):
    '''
    Disconnect from ODBC Source
    ----------
    [db_odbc.1]\n
    Disconnects an ODBC source connection.\n
    Arguments
    ----------
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TRANSACT [`choice`] : Transactions. Available Choices: [0] rollback [1] commit Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '1', 'Disconnect from ODBC Source')
    if Tool.is_Okay():
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TRANSACT', TRANSACT)
        return Tool.Execute(Verbose)
    return False

def CommitRollback_Transaction(
    CONNECTION=None,
    SOURCE=None,
    TRANSACT=None, Verbose=2):
    '''
    Commit/Rollback Transaction
    ----------
    [db_odbc.2]\n
    Execute a commit or rollback on open transactions with ODBC source.\n
    Arguments
    ----------
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - SOURCE [`choice`] : Source. Available Choices: Default: 0
    - TRANSACT [`choice`] : Transactions. Available Choices: [0] rollback [1] commit Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '2', 'Commit/Rollback Transaction')
    if Tool.is_Okay():
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('SOURCE', SOURCE)
        Tool.Set_Option('TRANSACT', TRANSACT)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_2(
    CONNECTION=None,
    SOURCE=None,
    TRANSACT=None, Verbose=2):
    '''
    Commit/Rollback Transaction
    ----------
    [db_odbc.2]\n
    Execute a commit or rollback on open transactions with ODBC source.\n
    Arguments
    ----------
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - SOURCE [`choice`] : Source. Available Choices: Default: 0
    - TRANSACT [`choice`] : Transactions. Available Choices: [0] rollback [1] commit Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '2', 'Commit/Rollback Transaction')
    if Tool.is_Okay():
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('SOURCE', SOURCE)
        Tool.Set_Option('TRANSACT', TRANSACT)
        return Tool.Execute(Verbose)
    return False

def Execute_SQL(
    CONNECTION=None,
    SQL=None,
    COMMIT=None,
    STOP=None, Verbose=2):
    '''
    Execute SQL
    ----------
    [db_odbc.3]\n
    Execute SQL commands on a connected ODBC source. Separate different commands with a semicolon (';').\n
    Arguments
    ----------
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - SQL [`long text`] : SQL Statement. Default: CREATE TABLE myTable1 (Col1 VARCHAR(255) PRIMARY KEY, Col2 INTEGER);
INSERT INTO myTable1 (Col1, Col2) VALUES('First Value', 1);
DROP TABLE myTable1;

    - COMMIT [`boolean`] : Commit. Default: 1
    - STOP [`boolean`] : Stop on Error. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '3', 'Execute SQL')
    if Tool.is_Okay():
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('SQL', SQL)
        Tool.Set_Option('COMMIT', COMMIT)
        Tool.Set_Option('STOP', STOP)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_3(
    CONNECTION=None,
    SQL=None,
    COMMIT=None,
    STOP=None, Verbose=2):
    '''
    Execute SQL
    ----------
    [db_odbc.3]\n
    Execute SQL commands on a connected ODBC source. Separate different commands with a semicolon (';').\n
    Arguments
    ----------
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - SQL [`long text`] : SQL Statement. Default: CREATE TABLE myTable1 (Col1 VARCHAR(255) PRIMARY KEY, Col2 INTEGER);
INSERT INTO myTable1 (Col1, Col2) VALUES('First Value', 1);
DROP TABLE myTable1;

    - COMMIT [`boolean`] : Commit. Default: 1
    - STOP [`boolean`] : Stop on Error. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '3', 'Execute SQL')
    if Tool.is_Okay():
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('SQL', SQL)
        Tool.Set_Option('COMMIT', COMMIT)
        Tool.Set_Option('STOP', STOP)
        return Tool.Execute(Verbose)
    return False

def List_Table_Fields(
    FIELDS=None,
    CONNECTION=None,
    TABLE=None, Verbose=2):
    '''
    List Table Fields
    ----------
    [db_odbc.4]\n
    Loads table information from ODBC data source.\n
    Arguments
    ----------
    - FIELDS [`output table`] : Fields
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TABLE [`choice`] : Table Source. Available Choices: Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '4', 'List Table Fields')
    if Tool.is_Okay():
        Tool.Set_Output('FIELDS', FIELDS)
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TABLE', TABLE)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_4(
    FIELDS=None,
    CONNECTION=None,
    TABLE=None, Verbose=2):
    '''
    List Table Fields
    ----------
    [db_odbc.4]\n
    Loads table information from ODBC data source.\n
    Arguments
    ----------
    - FIELDS [`output table`] : Fields
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TABLE [`choice`] : Table Source. Available Choices: Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '4', 'List Table Fields')
    if Tool.is_Okay():
        Tool.Set_Output('FIELDS', FIELDS)
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TABLE', TABLE)
        return Tool.Execute(Verbose)
    return False

def Import_Table(
    TABLE=None,
    CONNECTION=None,
    SOURCE=None, Verbose=2):
    '''
    Import Table
    ----------
    [db_odbc.5]\n
    Imports a table from a database via ODBC.\n
    Arguments
    ----------
    - TABLE [`output table`] : Table
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - SOURCE [`choice`] : Table Source. Available Choices: Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '5', 'Import Table')
    if Tool.is_Okay():
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('SOURCE', SOURCE)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_5(
    TABLE=None,
    CONNECTION=None,
    SOURCE=None, Verbose=2):
    '''
    Import Table
    ----------
    [db_odbc.5]\n
    Imports a table from a database via ODBC.\n
    Arguments
    ----------
    - TABLE [`output table`] : Table
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - SOURCE [`choice`] : Table Source. Available Choices: Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '5', 'Import Table')
    if Tool.is_Okay():
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('SOURCE', SOURCE)
        return Tool.Execute(Verbose)
    return False

def Export_Table(
    TABLE=None,
    CONNECTION=None,
    TABLE_PK=None,
    TABLE_NN=None,
    TABLE_UQ=None,
    NAME=None,
    EXISTS=None, Verbose=2):
    '''
    Export Table
    ----------
    [db_odbc.6]\n
    Exports a table to a database via ODBC.\n
    Arguments
    ----------
    - TABLE [`input table`] : Table
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TABLE_PK [`table fields`] : Primary Key
    - TABLE_NN [`table fields`] : Not Null
    - TABLE_UQ [`table fields`] : Unique
    - NAME [`text`] : Table Name
    - EXISTS [`choice`] : If table exists.... Available Choices: [0] abort export [1] replace existing table [2] append records, if table structure allows Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '6', 'Export Table')
    if Tool.is_Okay():
        Tool.Set_Input ('TABLE', TABLE)
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TABLE_PK', TABLE_PK)
        Tool.Set_Option('TABLE_NN', TABLE_NN)
        Tool.Set_Option('TABLE_UQ', TABLE_UQ)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('EXISTS', EXISTS)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_6(
    TABLE=None,
    CONNECTION=None,
    TABLE_PK=None,
    TABLE_NN=None,
    TABLE_UQ=None,
    NAME=None,
    EXISTS=None, Verbose=2):
    '''
    Export Table
    ----------
    [db_odbc.6]\n
    Exports a table to a database via ODBC.\n
    Arguments
    ----------
    - TABLE [`input table`] : Table
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TABLE_PK [`table fields`] : Primary Key
    - TABLE_NN [`table fields`] : Not Null
    - TABLE_UQ [`table fields`] : Unique
    - NAME [`text`] : Table Name
    - EXISTS [`choice`] : If table exists.... Available Choices: [0] abort export [1] replace existing table [2] append records, if table structure allows Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '6', 'Export Table')
    if Tool.is_Okay():
        Tool.Set_Input ('TABLE', TABLE)
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TABLE_PK', TABLE_PK)
        Tool.Set_Option('TABLE_NN', TABLE_NN)
        Tool.Set_Option('TABLE_UQ', TABLE_UQ)
        Tool.Set_Option('NAME', NAME)
        Tool.Set_Option('EXISTS', EXISTS)
        return Tool.Execute(Verbose)
    return False

def Drop_Table(
    CONNECTION=None,
    TABLE=None, Verbose=2):
    '''
    Drop Table
    ----------
    [db_odbc.7]\n
    Deletes a table from a database via ODBC.\n
    Arguments
    ----------
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TABLE [`choice`] : Table. Available Choices: Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '7', 'Drop Table')
    if Tool.is_Okay():
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TABLE', TABLE)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_7(
    CONNECTION=None,
    TABLE=None, Verbose=2):
    '''
    Drop Table
    ----------
    [db_odbc.7]\n
    Deletes a table from a database via ODBC.\n
    Arguments
    ----------
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TABLE [`choice`] : Table. Available Choices: Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '7', 'Drop Table')
    if Tool.is_Okay():
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TABLE', TABLE)
        return Tool.Execute(Verbose)
    return False

def Import_Table_from_Query(
    TABLE=None,
    CONNECTION=None,
    TABLES=None,
    FIELDS=None,
    WHERE=None,
    GROUP=None,
    HAVING=None,
    ORDER=None,
    DISTINCT=None, Verbose=2):
    '''
    Import Table from Query
    ----------
    [db_odbc.8]\n
    Import a table from a database through ODBC via SQL query.\n
    > SELECT [DISTINCT|ALL] 'Fields' FROM 'Tables' WHERE 'Where' [GROUP BY 'Group' [HAVING 'Having']] [ORDER BY 'Order']\n
    Arguments
    ----------
    - TABLE [`output table`] : Table
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TABLES [`text`] : Tables
    - FIELDS [`text`] : Fields. Default: *
    - WHERE [`text`] : Where
    - GROUP [`text`] : Group
    - HAVING [`text`] : Having
    - ORDER [`text`] : Order
    - DISTINCT [`boolean`] : Distinct. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '8', 'Import Table from Query')
    if Tool.is_Okay():
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TABLES', TABLES)
        Tool.Set_Option('FIELDS', FIELDS)
        Tool.Set_Option('WHERE', WHERE)
        Tool.Set_Option('GROUP', GROUP)
        Tool.Set_Option('HAVING', HAVING)
        Tool.Set_Option('ORDER', ORDER)
        Tool.Set_Option('DISTINCT', DISTINCT)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_8(
    TABLE=None,
    CONNECTION=None,
    TABLES=None,
    FIELDS=None,
    WHERE=None,
    GROUP=None,
    HAVING=None,
    ORDER=None,
    DISTINCT=None, Verbose=2):
    '''
    Import Table from Query
    ----------
    [db_odbc.8]\n
    Import a table from a database through ODBC via SQL query.\n
    > SELECT [DISTINCT|ALL] 'Fields' FROM 'Tables' WHERE 'Where' [GROUP BY 'Group' [HAVING 'Having']] [ORDER BY 'Order']\n
    Arguments
    ----------
    - TABLE [`output table`] : Table
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0
    - TABLES [`text`] : Tables
    - FIELDS [`text`] : Fields. Default: *
    - WHERE [`text`] : Where
    - GROUP [`text`] : Group
    - HAVING [`text`] : Having
    - ORDER [`text`] : Order
    - DISTINCT [`boolean`] : Distinct. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '8', 'Import Table from Query')
    if Tool.is_Okay():
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('CONNECTION', CONNECTION)
        Tool.Set_Option('TABLES', TABLES)
        Tool.Set_Option('FIELDS', FIELDS)
        Tool.Set_Option('WHERE', WHERE)
        Tool.Set_Option('GROUP', GROUP)
        Tool.Set_Option('HAVING', HAVING)
        Tool.Set_Option('ORDER', ORDER)
        Tool.Set_Option('DISTINCT', DISTINCT)
        return Tool.Execute(Verbose)
    return False

def List_Data_Sources(
    SOURCES=None,
    CONNECTED=None, Verbose=2):
    '''
    List Data Sources
    ----------
    [db_odbc.9]\n
    Lists all ODBC sources.\n
    Arguments
    ----------
    - SOURCES [`output table`] : Data Sources
    - CONNECTED [`boolean`] : Only List Connected Sources. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '9', 'List Data Sources')
    if Tool.is_Okay():
        Tool.Set_Output('SOURCES', SOURCES)
        Tool.Set_Option('CONNECTED', CONNECTED)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_9(
    SOURCES=None,
    CONNECTED=None, Verbose=2):
    '''
    List Data Sources
    ----------
    [db_odbc.9]\n
    Lists all ODBC sources.\n
    Arguments
    ----------
    - SOURCES [`output table`] : Data Sources
    - CONNECTED [`boolean`] : Only List Connected Sources. Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '9', 'List Data Sources')
    if Tool.is_Okay():
        Tool.Set_Output('SOURCES', SOURCES)
        Tool.Set_Option('CONNECTED', CONNECTED)
        return Tool.Execute(Verbose)
    return False

def List_Tables(
    TABLES=None,
    CONNECTION=None, Verbose=2):
    '''
    List Tables
    ----------
    [db_odbc.10]\n
    Lists all tables of an ODBC data source.\n
    Arguments
    ----------
    - TABLES [`output table`] : Tables
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '10', 'List Tables')
    if Tool.is_Okay():
        Tool.Set_Output('TABLES', TABLES)
        Tool.Set_Option('CONNECTION', CONNECTION)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_10(
    TABLES=None,
    CONNECTION=None, Verbose=2):
    '''
    List Tables
    ----------
    [db_odbc.10]\n
    Lists all tables of an ODBC data source.\n
    Arguments
    ----------
    - TABLES [`output table`] : Tables
    - CONNECTION [`choice`] : Connection. Available Choices: Default: 0

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '10', 'List Tables')
    if Tool.is_Okay():
        Tool.Set_Output('TABLES', TABLES)
        Tool.Set_Option('CONNECTION', CONNECTION)
        return Tool.Execute(Verbose)
    return False

def Disconnect_All(
    TRANSACT=None, Verbose=2):
    '''
    Disconnect All
    ----------
    [db_odbc.11]\n
    Disconnects all connected ODBC sources.\n
    Arguments
    ----------
    - TRANSACT [`choice`] : Transactions. Available Choices: [0] rollback [1] commit Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '11', 'Disconnect All')
    if Tool.is_Okay():
        Tool.Set_Option('TRANSACT', TRANSACT)
        return Tool.Execute(Verbose)
    return False

def run_tool_db_odbc_11(
    TRANSACT=None, Verbose=2):
    '''
    Disconnect All
    ----------
    [db_odbc.11]\n
    Disconnects all connected ODBC sources.\n
    Arguments
    ----------
    - TRANSACT [`choice`] : Transactions. Available Choices: [0] rollback [1] commit Default: 1

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('db_odbc', '11', 'Disconnect All')
    if Tool.is_Okay():
        Tool.Set_Option('TRANSACT', TRANSACT)
        return Tool.Execute(Verbose)
    return False




#! /usr/bin/env python

'''
Python Interface to SAGA Tools Library
----------
- Category : Simulation
- Name     : Cellular Automata
- ID       : sim_cellular_automata

Description
----------
Cellular Automata
'''

from PySAGA.helper import Tool_Wrapper

def Conways_Game_of_Life(
    TARGET_TEMPLATE=None,
    LIFE=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    REFRESH=None,
    FADECOLOR=None, Verbose=2):
    '''
    Conway's Game of Life
    ----------
    [sim_cellular_automata.0]\n
    Conway's Game of Life - a classical cellular automat.\n
    Arguments
    ----------
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - LIFE [`output grid`] : Life
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - REFRESH [`choice`] : Refresh. Available Choices: [0] no refresh [1] random [2] virus 1 [3] virus 2 Default: 1
    - FADECOLOR [`integer number`] : Fade Color Count. Minimum: 3 Maximum: 255 Default: 64

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_cellular_automata', '0', 'Conway\'s Game of Life')
    if Tool.is_Okay():
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('LIFE', LIFE)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('REFRESH', REFRESH)
        Tool.Set_Option('FADECOLOR', FADECOLOR)
        return Tool.Execute(Verbose)
    return False

def run_tool_sim_cellular_automata_0(
    TARGET_TEMPLATE=None,
    LIFE=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    REFRESH=None,
    FADECOLOR=None, Verbose=2):
    '''
    Conway's Game of Life
    ----------
    [sim_cellular_automata.0]\n
    Conway's Game of Life - a classical cellular automat.\n
    Arguments
    ----------
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - LIFE [`output grid`] : Life
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - REFRESH [`choice`] : Refresh. Available Choices: [0] no refresh [1] random [2] virus 1 [3] virus 2 Default: 1
    - FADECOLOR [`integer number`] : Fade Color Count. Minimum: 3 Maximum: 255 Default: 64

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_cellular_automata', '0', 'Conway\'s Game of Life')
    if Tool.is_Okay():
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('LIFE', LIFE)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('REFRESH', REFRESH)
        Tool.Set_Option('FADECOLOR', FADECOLOR)
        return Tool.Execute(Verbose)
    return False

def WaTor(
    TARGET_TEMPLATE=None,
    GRID=None,
    TABLE=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    REFRESH=None,
    INIT_FISH=None,
    INIT_SHARK=None,
    FISH_BIRTH=None,
    SHARK_BIRTH=None,
    SHARK_STARVE=None, Verbose=2):
    '''
    Wa-Tor
    ----------
    [sim_cellular_automata.1]\n
    Wa-Tor - an ecological simulation of predator-prey populations - is based upon A. K. Dewdney's 'Computer Recreations' article in the December 1984 issue of Scientific American.\n
    Arguments
    ----------
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - GRID [`output grid`] : Wa-Tor
    - TABLE [`output table`] : Cycles
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - REFRESH [`boolean`] : Refresh. Default: 1
    - INIT_FISH [`floating point number`] : Initial Number of Fishes [%]. Minimum: 0.000000 Maximum: 100.000000 Default: 30.000000
    - INIT_SHARK [`floating point number`] : Initial Number of Sharks [%]. Minimum: 0.000000 Maximum: 100.000000 Default: 7.500000
    - FISH_BIRTH [`integer number`] : Birth Rate of Fishes. Minimum: 0 Default: 3
    - SHARK_BIRTH [`integer number`] : Birth Rate of Sharks. Minimum: 0 Default: 12
    - SHARK_STARVE [`integer number`] : Max. Starvation Time for Sharks. Minimum: 0 Default: 4

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_cellular_automata', '1', 'Wa-Tor')
    if Tool.is_Okay():
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('REFRESH', REFRESH)
        Tool.Set_Option('INIT_FISH', INIT_FISH)
        Tool.Set_Option('INIT_SHARK', INIT_SHARK)
        Tool.Set_Option('FISH_BIRTH', FISH_BIRTH)
        Tool.Set_Option('SHARK_BIRTH', SHARK_BIRTH)
        Tool.Set_Option('SHARK_STARVE', SHARK_STARVE)
        return Tool.Execute(Verbose)
    return False

def run_tool_sim_cellular_automata_1(
    TARGET_TEMPLATE=None,
    GRID=None,
    TABLE=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    REFRESH=None,
    INIT_FISH=None,
    INIT_SHARK=None,
    FISH_BIRTH=None,
    SHARK_BIRTH=None,
    SHARK_STARVE=None, Verbose=2):
    '''
    Wa-Tor
    ----------
    [sim_cellular_automata.1]\n
    Wa-Tor - an ecological simulation of predator-prey populations - is based upon A. K. Dewdney's 'Computer Recreations' article in the December 1984 issue of Scientific American.\n
    Arguments
    ----------
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - GRID [`output grid`] : Wa-Tor
    - TABLE [`output table`] : Cycles
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - REFRESH [`boolean`] : Refresh. Default: 1
    - INIT_FISH [`floating point number`] : Initial Number of Fishes [%]. Minimum: 0.000000 Maximum: 100.000000 Default: 30.000000
    - INIT_SHARK [`floating point number`] : Initial Number of Sharks [%]. Minimum: 0.000000 Maximum: 100.000000 Default: 7.500000
    - FISH_BIRTH [`integer number`] : Birth Rate of Fishes. Minimum: 0 Default: 3
    - SHARK_BIRTH [`integer number`] : Birth Rate of Sharks. Minimum: 0 Default: 12
    - SHARK_STARVE [`integer number`] : Max. Starvation Time for Sharks. Minimum: 0 Default: 4

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_cellular_automata', '1', 'Wa-Tor')
    if Tool.is_Okay():
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Output('TABLE', TABLE)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('REFRESH', REFRESH)
        Tool.Set_Option('INIT_FISH', INIT_FISH)
        Tool.Set_Option('INIT_SHARK', INIT_SHARK)
        Tool.Set_Option('FISH_BIRTH', FISH_BIRTH)
        Tool.Set_Option('SHARK_BIRTH', SHARK_BIRTH)
        Tool.Set_Option('SHARK_STARVE', SHARK_STARVE)
        return Tool.Execute(Verbose)
    return False

def Hodgepodge_Machine(
    TARGET_TEMPLATE=None,
    GRID=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    REFRESH=None,
    RADIUS=None,
    NSTATES=None,
    TSICK=None,
    TINFECTED=None,
    SPEED=None, Verbose=2):
    '''
    Hodgepodge Machine
    ----------
    [sim_cellular_automata.2]\n
    The hodgepodge machine.\n
    Arguments
    ----------
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - GRID [`output grid`] : Hodgepodge
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - REFRESH [`boolean`] : Refresh. Default: 1
    - RADIUS [`integer number`] : Radius. Minimum: 1 Default: 1
    - NSTATES [`integer number`] : Number of Infection States. Minimum: 1 Default: 150
    - TSICK [`integer number`] : Threshold Sickness. Minimum: 1 Default: 1
    - TINFECTED [`integer number`] : Threshold Infected. Minimum: 1 Default: 3
    - SPEED [`integer number`] : Infection Speed. Minimum: 1 Default: 23

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_cellular_automata', '2', 'Hodgepodge Machine')
    if Tool.is_Okay():
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('REFRESH', REFRESH)
        Tool.Set_Option('RADIUS', RADIUS)
        Tool.Set_Option('NSTATES', NSTATES)
        Tool.Set_Option('TSICK', TSICK)
        Tool.Set_Option('TINFECTED', TINFECTED)
        Tool.Set_Option('SPEED', SPEED)
        return Tool.Execute(Verbose)
    return False

def run_tool_sim_cellular_automata_2(
    TARGET_TEMPLATE=None,
    GRID=None,
    TARGET_DEFINITION=None,
    TARGET_USER_SIZE=None,
    TARGET_USER_XMIN=None,
    TARGET_USER_XMAX=None,
    TARGET_USER_YMIN=None,
    TARGET_USER_YMAX=None,
    TARGET_USER_COLS=None,
    TARGET_USER_ROWS=None,
    TARGET_USER_FLAT=None,
    TARGET_USER_FITS=None,
    TARGET_SYSTEM=None,
    REFRESH=None,
    RADIUS=None,
    NSTATES=None,
    TSICK=None,
    TINFECTED=None,
    SPEED=None, Verbose=2):
    '''
    Hodgepodge Machine
    ----------
    [sim_cellular_automata.2]\n
    The hodgepodge machine.\n
    Arguments
    ----------
    - TARGET_TEMPLATE [`optional input grid`] : Target System. use this grid's system for output grids
    - GRID [`output grid`] : Hodgepodge
    - TARGET_DEFINITION [`choice`] : Target Grid System. Available Choices: [0] user defined [1] grid or grid system Default: 0
    - TARGET_USER_SIZE [`floating point number`] : Cellsize. Minimum: 0.000000 Default: 1.000000
    - TARGET_USER_XMIN [`floating point number`] : West. Default: 0.000000
    - TARGET_USER_XMAX [`floating point number`] : East. Default: 100.000000
    - TARGET_USER_YMIN [`floating point number`] : South. Default: 0.000000
    - TARGET_USER_YMAX [`floating point number`] : North. Default: 100.000000
    - TARGET_USER_COLS [`integer number`] : Columns. Minimum: 1 Default: 101 Number of cells in East-West direction.
    - TARGET_USER_ROWS [`integer number`] : Rows. Minimum: 1 Default: 101 Number of cells in North-South direction.
    - TARGET_USER_FLAT [`boolean`] : Rounding. Default: 1 Round bounding coordinates to multiples of cell size. Ignored if cell size has decimal places.
    - TARGET_USER_FITS [`choice`] : Fit. Available Choices: [0] nodes [1] cells Default: 0
    - TARGET_SYSTEM [`grid system`] : Grid System
    - REFRESH [`boolean`] : Refresh. Default: 1
    - RADIUS [`integer number`] : Radius. Minimum: 1 Default: 1
    - NSTATES [`integer number`] : Number of Infection States. Minimum: 1 Default: 150
    - TSICK [`integer number`] : Threshold Sickness. Minimum: 1 Default: 1
    - TINFECTED [`integer number`] : Threshold Infected. Minimum: 1 Default: 3
    - SPEED [`integer number`] : Infection Speed. Minimum: 1 Default: 23

    - Verbose [`integer number`] : Verbosity level, 0=silent, 1=tool name and success notification, 2=complete tool output.\n
    Returns
    ----------
    `boolean` : `True` on success, `False` on failure.
    '''
    Tool = Tool_Wrapper('sim_cellular_automata', '2', 'Hodgepodge Machine')
    if Tool.is_Okay():
        Tool.Set_Input ('TARGET_TEMPLATE', TARGET_TEMPLATE)
        Tool.Set_Output('GRID', GRID)
        Tool.Set_Option('TARGET_DEFINITION', TARGET_DEFINITION)
        Tool.Set_Option('TARGET_USER_SIZE', TARGET_USER_SIZE)
        Tool.Set_Option('TARGET_USER_XMIN', TARGET_USER_XMIN)
        Tool.Set_Option('TARGET_USER_XMAX', TARGET_USER_XMAX)
        Tool.Set_Option('TARGET_USER_YMIN', TARGET_USER_YMIN)
        Tool.Set_Option('TARGET_USER_YMAX', TARGET_USER_YMAX)
        Tool.Set_Option('TARGET_USER_COLS', TARGET_USER_COLS)
        Tool.Set_Option('TARGET_USER_ROWS', TARGET_USER_ROWS)
        Tool.Set_Option('TARGET_USER_FLAT', TARGET_USER_FLAT)
        Tool.Set_Option('TARGET_USER_FITS', TARGET_USER_FITS)
        Tool.Set_Option('TARGET_SYSTEM', TARGET_SYSTEM)
        Tool.Set_Option('REFRESH', REFRESH)
        Tool.Set_Option('RADIUS', RADIUS)
        Tool.Set_Option('NSTATES', NSTATES)
        Tool.Set_Option('TSICK', TSICK)
        Tool.Set_Option('TINFECTED', TINFECTED)
        Tool.Set_Option('SPEED', SPEED)
        return Tool.Execute(Verbose)
    return False




#_______________________________________________________#
#########################################################
#                                                       #
#          Copyright (C) 2020 by Olaf Conrad            #
#                                                       #
#_______________________________________________________#
#                                                       #
# This file is part of 'SAGA - System for Automated     #
# Geoscientific Analyses'.                              #
#                                                       #
# This library is free software; you can redistribute   #
# it and/or modify it under the terms of the GNU Lesser #
# General Public License as published by the Free       #
# Software Foundation, either version 2.1 of the        #
# License, or (at your option) any later version.       #
#                                                       #
# This library is distributed in the hope that it will  #
# be useful, but WITHOUT ANY WARRANTY; without even the #
# implied warranty of MERCHANTABILITY or FITNESS FOR A  #
# PARTICULAR PURPOSE. See the GNU Lesser General Public #
# License for more details.                             #
#                                                       #
# You should have received a copy of the GNU Lesser     #
# General Public License along with this program; if    #
# not, see <http://www.gnu.org/licenses/>.              #
#                                                       #
#_______________________________________________________#
#                                                       #
#    contact:    Olaf Conrad                            #
#                Institute of Geography                 #
#                University of Hamburg                  #
#                Germany                                #
#                                                       #
#    e-mail:     oconrad@saga-gis.org                   #
#                                                       #
#########################################################

#########################################################
#________________________________________________________
import sys, os, glob, subprocess, arcpy, ConversionUtils, shutil


#########################################################
# Globals
#________________________________________________________
DIR_LOG  = None
DIR_LOG  = os.path.dirname(__file__) # uncomment to create logs in toolbox directory

CREATE_NO_WINDOW = 0x08000000


#########################################################
# File Tools
#________________________________________________________
def File_Get_TempName(Extension):
	if 'tempnam' in dir(os): # sys.version_info.major < 3: # ArcGIS Desktop
		return os.tempnam(None, 'arc_saga_') + '.' + Extension
	import tempfile
	return tempfile._get_default_tempdir() + os.sep + 'arcsaga_' + next(tempfile._get_candidate_names()) + '.' + Extension

#________________________________________________________
def File_Cmp_Extension(File, Extension):
	if File != '#' and File != None:
		File, ext = os.path.splitext(File)
		if '.' + Extension == ext:
			return True
	return False

#________________________________________________________
def File_Set_Extension(File, Extension):
	if File != '#' and File != None:
		File, ext = os.path.splitext(File)
		File += '.' + Extension
		return File
	return None

#________________________________________________________
def File_Remove_All(File_Name):
	if File_Name != '#' and File_Name != None:
		Files = File_Set_Extension(File_Name, '*')
		for File in glob.glob(Files):
			os.remove(File)
	return


#########################################################
# The SAGA Tool Execution Class
#________________________________________________________
class SAGA_Tool:

	#####################################################
	# Construction
	#____________________________________________________
	def __init__(self, Library, Tool):
		self.Library    = Library
		self.Tool       = Tool
		self.Parameters	= None
		self.Temporary  = None
		self.Output     = None

		saga_cmd = None
	#	saga_cmd = 'F:/develop/saga/saga-code/master/saga-gis/bin/saga_vc_x64/saga_cmd.exe'	# define hard coded path to saga_cmd.exe
		if saga_cmd == None:
			head, tail = os.path.split(os.path.dirname(__file__))
			saga_cmd = head + os.sep + 'saga_cmd.exe'

		if os.path.isfile(saga_cmd) == True:
			self.saga_cmd =  saga_cmd	# use SAGA instance of this installation
		else:
			self.saga_cmd = 'saga_cmd'	# assuming saga_cmd is included in the PATH environment variable


	#####################################################
	# Execution
	#____________________________________________________
	def Run(self, bIgnoreLog = False):
		cmd = '\"' + self.saga_cmd + '\" -f=q ' + self.Library + ' ' + self.Tool
		for Item in self.Parameters:
			cmd += ' ' + Item

		if bIgnoreLog == False:
			arcpy.AddMessage('_________________________\n')
			arcpy.AddMessage(cmd)

		if bIgnoreLog == False and DIR_LOG != None:
			cmd_out = open(DIR_LOG + os.sep + 'arcsaga.log'      , 'w')
			cmd_err = open(DIR_LOG + os.sep + 'arcsaga.error.log', 'w')
			cmd_out.write('\n___\nPython-' + sys.version + '\n___\n' + cmd)	# print to log file
			Result = subprocess.call(cmd, creationflags=CREATE_NO_WINDOW, stdout=cmd_out, stderr=cmd_err)
		else:
			Result = subprocess.call(cmd, creationflags=CREATE_NO_WINDOW)

		if Result != 0:
			arcpy.AddMessage('_________________________\n')
			arcpy.AddMessage(cmd)
			arcpy.AddError('...failed to run SAGA tool!')

		elif self.Output != None:
			for Data in self.Output:
				self.Get_Output(Data[0], Data[1], Data[2])

		if self.Temporary != None:
			for Data in self.Temporary:
				File_Remove_All(Data)

		return Result


	#####################################################
	# Data Lists
	#____________________________________________________
	def Add_Output(self, Identifier, File, Type):
		if self.Output == None:
			self.Output  = [[Identifier, File, Type]]
		else:
			self.Output += [[Identifier, File, Type]]
		return

	#____________________________________________________
	def Add_Temporary(self, File):
		if self.Temporary == None:
			self.Temporary  = [File]
		else:
			self.Temporary += [File]
		return

	#____________________________________________________
	def Get_Temporary(self, Extension):
		File = File_Get_TempName(Extension)
		self.Add_Temporary(File)
		return File


	#####################################################
	# Parameters
	#____________________________________________________
	def Set_Option(self, Identifier, Value):
		if Value != '#' and Value != None:
			Value = '\"' + Value.strip() + '\"'
			if self.Parameters == None:
				self.Parameters  = ['-' + Identifier, Value]
			else:
				self.Parameters += ['-' + Identifier, Value]
		return

	#____________________________________________________
	def Set_Input(self, Identifier, Value, Type):
		Files = None

		if Value != '#' and Value != None:
			List  = ConversionUtils.SplitMultiInputs(Value)
			for Item in List:
				if   Type == 'grid'   or Type == 'grid_list':
					File = Arc_To_SAGA_Raster (Item)
				elif Type == 'table'  or Type == 'table_list':
					File = Arc_To_SAGA_Table  (Item)
				elif Type == 'shapes' or Type == 'shapes_list':
					File = Arc_To_SAGA_Feature(Item)
				elif Type == 'points' or Type == 'points_list':
					File = Arc_To_SAGA_Points (Item)
				else:
					File = None

				if File != None:
					if Files == None:
						Files = File
					else:
						Files = Files + ';' + File

					if File != Item:
						self.Add_Temporary(File)

		if Files != None:
			self.Set_Option(Identifier, Files)

		return Files

	#____________________________________________________
	def Set_Output(self, Identifier, Value, Type):
		Files = None

		if Value != '#' and Value != None:
			List  = ConversionUtils.SplitMultiInputs(Value)
			for Item in List:
				if   Type == 'grid'   or Type == 'grid_list':
					if File_Cmp_Extension(Item,   'sdat') == False:
						File = self.Get_Temporary('sgrd')
					else:
						File = File_Set_Extension(Item, 'sgrd')
				elif Type == 'table'  or Type == 'table_list':
					if File_Cmp_Extension(Item,   'dbf') == False:
						File = self.Get_Temporary('dbf')
					else:
						File = Item
				elif Type == 'shapes' or Type == 'shapes_list':
					if File_Cmp_Extension(Item,   'shp') == False:
						File = self.Get_Temporary('shp')
					else:
						File = Item
				elif Type == 'points' or Type == 'points_list':
					File = self.Get_Temporary('spc')
				else:
					File = None

				if File != None:
					if Files == None:
						Files = File
					else:
						Files = Files + ';' + File

		if Files != None:
			self.Set_Option(Identifier, Files)
			self.Add_Output(Files, Value, Type)

		return Files

	#____________________________________________________
	def Get_Output(self, SagaFile, ArcFile, Type):
		if ArcFile != '#' and ArcFile != None:
			List  = ArcFile.split(';')
			for Item in List:
				if   Type == 'grid'   or Type == 'grid_list':
					SAGA_To_Arc_Raster (SagaFile, Item)
				elif Type == 'table'  or Type == 'table_list':
					SAGA_To_Arc_Table  (SagaFile, Item)
				elif Type == 'shapes' or Type == 'shapes_list':
					SAGA_To_Arc_Feature(SagaFile, Item)
				elif Type == 'points' or Type == 'points_list':
					SAGA_To_Arc_Points (SagaFile, Item)
		return


#########################################################
# Raster Conversion
#________________________________________________________
def Arc_To_SAGA_Raster(Raster):
	Supported = ['sdat', 'tif', 'img', 'asc']
	for ext in Supported:
		if File_Cmp_Extension(Raster, ext) and os.path.isfile(Raster):
			return Raster

	File   = File_Get_TempName('tif')
	Raster = ConversionUtils.ValidateInputRaster(Raster)
	ConversionUtils.CopyRasters(Raster, File, "")

	return File

#________________________________________________________
def SAGA_To_Arc_Raster(File, Raster):
	if File_Cmp_Extension(Raster, 'sdat') == False: # conversion needed
		Tool = SAGA_Tool('io_gdal', '2') # 'Export Raster to GeoTIFF'
		Tool.Set_Option('GRIDS', File)

		if File_Cmp_Extension(Raster, 'tif'):
			arcpy.AddMessage('Output GeoTIFF: ' + File + ' >> ' + Raster)
			Tool.Set_Option('FILE', Raster)
			if Tool.Run(True) != 0:
				return False
		else:
			arcpy.AddMessage('Output Convert: ' + File + ' >> ' + Raster)
			GeoTIFF = File_Set_Extension(File, 'tif')
			Tool.Set_Option('FILE', GeoTIFF)
			if Tool.Run(True) != 0:
				return False
			GeoTIFF = ConversionUtils.ValidateInputRaster(GeoTIFF)
			ConversionUtils.CopyRasters(GeoTIFF, Raster, "")

	Arc_Load_Layer(Raster)
	
	return True


#########################################################
# Feature Conversion
#________________________________________________________
def Arc_To_SAGA_Feature(Feature):
	if File_Cmp_Extension(Feature, 'shp') == True:
		return Feature

	File = File_Get_TempName('.shp')
	ConversionUtils.CopyFeatures(Feature, File)

	return File

#________________________________________________________
def SAGA_To_Arc_Feature(File, Feature):
	if File_Cmp_Extension(Feature, 'shp') == False:
		ConversionUtils.CopyFeatures(File, Feature)
		File_Remove_All(File)

	Arc_Load_Layer(Feature)

	return True


#########################################################
# Table Conversion
#________________________________________________________
def Arc_To_SAGA_Table(Table):
	if File_Cmp_Extension(Table, 'dbf') == True:
		return Table

	File  = File_Get_TempName('.dbf')
	ConversionUtils.CopyRows(Table, File)

	return File

#________________________________________________________
def SAGA_To_Arc_Table(File, Table):
	if File_Cmp_Extension(Table, 'dbf') == False:
		ConversionUtils.CopyRows(File, Table)
		File_Remove_All(File)

	return True

	
#########################################################
# ArcMap Interaction
#________________________________________________________
def Arc_Load_Layer(Layer):
	if Layer == '#' or not Layer:
		return

	#____________________________________________________
	if sys.version_info.major < 3: # ArcGIS Desktop
		Map_Project = arcpy.mapping.MapDocument("CURRENT")
		Map_Frame   = arcpy.mapping.ListDataFrames(Map_Project)[0]
		Map_Layer   = arcpy.mapping.Layer(Layer)
		if Map_Layer.isRasterLayer:
			File = os.path.dirname(__file__) + os.sep + 'grid.lyr'
			if os.path.isfile(File) == True:
				Src_Layer = arcpy.mapping.Layer(File)
				arcpy.mapping.UpdateLayer(Map_Frame, Map_Layer, Src_Layer, True)
		arcpy.mapping.AddLayer(Map_Frame, Map_Layer, 'AUTO_ARRANGE')
		return

	#____________________________________________________
	Project = arcpy.mp.ArcGISProject("CURRENT")
	Map     = Project.listMaps()[0]
	Layer   = Map.addDataFromPath(Layer)
	return


#########################################################
#________________________________________________________
